'
' Blending Based Bokeh Render
' Version 1.0
' 
' Petr Schreiber, 2010
'

Type tBokehInfo       
  numImages     As Single
  radialOffset  As Single

  textureID     As Long
  displayListID As Long
  Width         As Long
  Height        As Long                     
End Type                                                     

Sub BokehRender_Init(bokehInfo As tBokehInfo, textureID As Long, width As Long, height As Long, radialOffset As Single, numImages As Single)                                                      
  
  bokehInfo.numImages    = numImages
  bokehInfo.radialOffset = radialOffset   
  
  bokehInfo.textureID = textureID
             
  ' You need to create variable to which we will pass data later 
  Dim texInfo As TBGL_tTexturingInfo 
  
  ' To fill variable fields, just use following   
  TBGL_TexturingQuery( texInfo ) 
                         
  ' Use the passed variable for tests in your code 
  If texInfo.NPOTSupport = %TRUE Then
    ' -- Grab screen at full detail
    bokehInfo.Width = width
    bokehInfo.Height= height   
  Else 
    ' -- Find the closest match
    TBGL_EvaluatePOTMatch( width, height, bokehInfo.Width, bokehInfo.Height)
  End If

  ' -- Make the target texture safely at checked size             
  TBGL_MakeTexture (String$ ( bokehInfo.Width*bokehInfo.Height*3, 255 ), 
  %TBGL_DATA_RGB,
   bokehInfo.Width, bokehInfo.Height, bokehInfo.textureID, 
   %TBGL_TEX_LINEAR   )
  
  ' -- Cache the image polygon
  Dim ListID(1) As Long
  TBGL_NewListSpace(ListID(1))
  
  bokehInfo.displayListID = ListID(1)     
  
  TBGL_NewList bokehInfo.displayListID
    TBGL_BeginPoly(%TBGL_QUADS)
      TBGL_TexCoord2D 0,0
      TBGL_Vertex -1,-1
      
      TBGL_TexCoord2D 1,0
      TBGL_Vertex  1,-1
  
      TBGL_TexCoord2D 1,1
      TBGL_Vertex  1, 1
  
      TBGL_TexCoord2D 0,1
      TBGL_Vertex -1,1        
    TBGL_EndPoly
  TBGL_EndList
                   
End Sub                               

Sub BokehRender_BeginCapture(bokehInfo As tBokehInfo)
  ' -- Set viewport to texture size
  TBGL_Viewport(0, 0, bokehInfo.Width, bokehInfo.Height, %TBGL_PARAM_PIXELS)
  ' -- But maintain original window ratio for perspective
  TBGL_RenderMatrix3D(%TBGL_CLIENTAREA)
End Sub

Sub BokehRender_EndCapture(bokehInfo As tBokehInfo)
  ' -- Capture the image
  TBGL_RenderToTexture( bokehInfo.textureID, 0, 0, bokehInfo.Width, bokehInfo.Height )
  
  ' -- Set viewport to original size
  TBGL_Viewport(0, 0, 1, 1, %TBGL_PARAM_RELATIVE)
  TBGL_RenderMatrix3D
End Sub
                                                                 
Sub BokehRender_Draw(bokehInfo As tBokehInfo)               
  Static angle, angleStep As Single
  Static brightness As Byte      
                                       
  ' -- Set proper blending mode
  TBGL_BlendFunc(%GL_ONE, %GL_ONE)                                       
  
                                       
  ' -- Set 2D rendering mode for the layers
  TBGL_RenderMatrix2D(-1,-1,1,1)
  TBGL_ClearFrame        

  ' -- Disable and set necessary states  
  TBGL_PushStateProtect(%TBGL_LIGHTING Or %TBGL_DEPTHMASK)
    TBGL_PushState(%TBGL_TEXTURING Or %TBGL_BLEND)
    
    TBGL_BindTexture(bokehInfo.textureID)
    
    brightness = 255/bokehInfo.numImages
    TBGL_Color(brightness, brightness, brightness)
    
    ' -- Render the images radially distributed around the center of screen
    angleStep = 360/bokehInfo.numImages 
    For angle = 1 To 360 Step angleStep    
      TBGL_PushMatrix                       
        TBGL_Rotate(angle)
        TBGL_Translate(0.001*bokehInfo.radialOffset, 0)
        TBGL_Rotate(-angle)               
        ' -- Polygon to hold image          
        TBGL_CallList(bokehInfo.displayListID)
      TBGL_PopMatrix
    Next       
    
    TBGL_PopState    
  TBGL_PopStateProtect   
  
  TBGL_RenderMatrix3D                                                   

End Sub             